@@ -62,7 +62,7 @@ __KERNEL_RCSID(0, "$NetBSD: vpci.c,v 1.6 2015/11/27 13:51:57 joerg Exp $");
 #define VDB_CONF_READ        0x08
 #define VDB_CONF_WRITE       0x10
 #define VDB_CONF             VDB_CONF_READ|VDB_CONF_WRITE
-int vpci_debug = 0x00;
+int vpci_debug = 0x04;
 #define DPRINTF(l, s)   do { if (vpci_debug & l) printf s; } while (0)
 #else
 #define DPRINTF(l, s)
@@ -114,7 +114,7 @@ void vpci_conf_write(pci_chipset_tag_t, pcitag_t, int, pcireg_t);
 static void * vpci_pci_intr_establish(pci_chipset_tag_t pc,
 				      pci_intr_handle_t ih, int level,
 				      int (*func)(void *), void *arg);
-
+void vpci_intr_ack(struct intrhand *);
 int vpci_intr_map(const struct pci_attach_args *, pci_intr_handle_t *);
 int vpci_bus_map(bus_space_tag_t, bus_addr_t,
     bus_size_t, int, vaddr_t, bus_space_handle_t *);
@@ -606,12 +606,56 @@ vpci_intr_establish(bus_space_tag_t t, int ihandle, int level,
 	ih->ih_pil = level;
 	ih->ih_number = ino;
 	ih->ih_pending = 0;
+	ih->ih_clr = NULL;
+	ih->ih_ack = vpci_intr_ack;
 	
 	intr_establish(ih->ih_pil, level != IPL_VM, ih);
 
+	uint64_t sysino = INTVEC(ihandle);
+	DPRINTF(VDB_INTR, ("vpci_intr_establish(): sysino 0x%lx\n", sysino));
+
+	int err;
+
+	err = hv_intr_settarget(sysino, cpus->ci_cpuid); /* FIXME curcpu-> */
+	if (err != H_EOK) {
+	  printf("hv_intr_settarget() failed - err = %d\n", err);
+#if 0
+	  return (NULL);
+#endif
+	} 
+
+	/* Clear pending interrupts. */
+	err = hv_intr_setstate(sysino, INTR_IDLE);
+	if (err != H_EOK) {
+	  printf("hv_intr_setstate() failed - err = %d\n", err);
+#if 0
+	  return (NULL);
+#endif
+	}
+
+	err = hv_intr_setenabled(sysino, INTR_ENABLED);
+	if (err != H_EOK) {
+	  printf("hv_intr_setenabled() failed - err = %d\n", err);
+#if 0
+	  return (NULL);
+#endif
+	}
+
+	DPRINTF(VDB_INTR, ("%s() returning %p\n", __func__, ih));
 	return (ih);
 }
 
+void
+vpci_intr_ack(struct intrhand *ih)
+{
+	int err;
+  /* FIXME check that ih_number is sysino */  
+	err = hv_intr_setstate(ih->ih_number, INTR_IDLE);
+	if (err != H_EOK) {
+		printf("hv_intr_setstate() failed - err = %d\n", err); /* FIXME panic() - is printf ok in intr context? */
+	}
+}
+
 static void *
 vpci_pci_intr_establish(pci_chipset_tag_t pc, pci_intr_handle_t ih, int level,
 	int (*func)(void *), void *arg)
