@@ -1,4 +1,4 @@
-/*	$NetBSD: cpu.c,v 1.127 2015/09/06 23:48:39 nakayama Exp $ */
+/*	$NetBSD: cpu.c,v 1.128 2016/04/17 14:32:03 martin Exp $ */
 
 /*
  * Copyright (c) 1996
@@ -52,7 +52,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: cpu.c,v 1.127 2015/09/06 23:48:39 nakayama Exp $");
+__KERNEL_RCSID(0, "$NetBSD: cpu.c,v 1.128 2016/04/17 14:32:03 martin Exp $");
 
 #include "opt_multiprocessor.h"
 
@@ -597,26 +597,35 @@ cpu_attach(device_t parent, device_t dev, void *aux)
 		paddr_t pa = ci->ci_paddr;
 		int err;
 
+		printf("ci_pa = %p\n", (void*)pa);
 		pa += CPUINFO_VA - INTSTACK;
-		pa += PAGE_SIZE;
+		printf("ci_pa = %p\n", (void*)pa);
+		pa += PAGE_SIZE; /* jump to somewhere after cpuinfo */
+		printf("ci_pa = %p\n", (void*)pa);
 
 		ci->ci_cpumq = pa;
+		printf("ci_cpumq = %p\n", (void*)ci->ci_cpumq);
 		err = hv_cpu_qconf(CPU_MONDO_QUEUE, ci->ci_cpumq, SUN4V_MONDO_QUEUE_SIZE);
 		if (err != H_EOK)
 			panic("Unable to set cpu mondo queue: %d", err);
 		pa += SUN4V_MONDO_QUEUE_SIZE * SUN4V_QUEUE_ENTRY_SIZE;
 		
 		ci->ci_devmq = pa;
+		printf("ci_devmq = %p\n", (void*)ci->ci_devmq);
 		err = hv_cpu_qconf(DEVICE_MONDO_QUEUE, ci->ci_devmq, SUN4V_MONDO_QUEUE_SIZE);
 		if (err != H_EOK)
 			panic("Unable to set device mondo queue: %d", err);
 		pa += SUN4V_MONDO_QUEUE_SIZE * SUN4V_QUEUE_ENTRY_SIZE;
 		
 		ci->ci_mondo = pa;
+		printf("ci_mondo = %p\n", (void*)ci->ci_mondo);
 		pa += 64; /* mondo message is 64 bytes */
 		
 		ci->ci_cpuset = pa;
+		printf("ci_cpuset = %p\n", (void*)ci->ci_cpuset);
 		pa += 64;
+
+		printf("FIXME check at devmq omraade faktisk er allokeret og tilgaengeligt...\n");
 	}
 	
 }
@@ -670,10 +679,6 @@ cpu_boot_secondary_processors(void)
 		return;
 	}
 
-	/* No MP for SUN4V yet */
-	if (CPU_ISSUN4V)
-		return;
-	
 	for (ci = cpus; ci != NULL; ci = ci->ci_next) {
 		if (ci->ci_cpuid == cpu_myid())
 			continue;
@@ -690,9 +695,12 @@ cpu_boot_secondary_processors(void)
 
 		int rc = prom_startcpu_by_cpuid(ci->ci_cpuid,
 		    (void *)cpu_spinup_trampoline, 0);
-		if (rc == -1)
+		if (rc == -1) {
+		  printf("cpu%d starting by cpuid failed\n", ci->ci_cpuid);
 			prom_startcpu(ci->ci_node,
 			    (void *)cpu_spinup_trampoline, 0);
+		}
+		printf("cpu%d starting initiated\n", ci->ci_cpuid);
 
 		for (i = 0; i < 2000; i++) {
 			membar_Sync();
@@ -714,7 +722,9 @@ cpu_boot_secondary_processors(void)
 		setpstate(pstate);
 
 		if (!CPUSET_HAS(cpus_active, ci->ci_index))
-			printf("cpu%d: startup failed\n", ci->ci_cpuid);
+			printf("cpu%d startup failed\n", ci->ci_cpuid); 
+		else
+			printf("cpu%d startup ok\n", ci->ci_cpuid); 
 	}
 }
 
