@@ -1,4 +1,4 @@
-/*	$NetBSD: pmap.c,v 1.301 2016/02/03 20:33:52 palle Exp $	*/
+/*	$NetBSD: pmap.c,v 1.302 2016/04/17 14:32:03 martin Exp $	*/
 /*
  *
  * Copyright (C) 1996-1999 Eduardo Horvath.
@@ -26,7 +26,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: pmap.c,v 1.301 2016/02/03 20:33:52 palle Exp $");
+__KERNEL_RCSID(0, "$NetBSD: pmap.c,v 1.302 2016/04/17 14:32:03 martin Exp $");
 
 #undef	NO_VCACHE /* Don't forget the locked TLB in dostart */
 #define	HWREF
@@ -49,6 +49,7 @@ __KERNEL_RCSID(0, "$NetBSD: pmap.c,v 1.301 2016/02/03 20:33:52 palle Exp $");
 #include <sys/cpu.h>
 
 #include <sys/exec_aout.h>	/* for MID_* */
+#include <sys/reboot.h>
 
 #include <uvm/uvm.h>
 
@@ -728,6 +729,10 @@ pmap_bootstrap(u_long kernelstart, u_long kernelend)
 
 	BDPRINTF(PDB_BOOT, ("Entered pmap_bootstrap.\n"));
 
+	/* XXX - incomplete spinup code for SUN4V */
+	if (CPU_ISSUN4V)
+		boothowto |= RB_MD1;
+
 	cache_setup_funcs();
 
 	/*
@@ -890,6 +895,7 @@ pmap_bootstrap(u_long kernelstart, u_long kernelend)
 		prom_printf("Cannot allocate cpu_infos\n");
 		prom_halt();
 	}
+	printf("pmap: cpu0paddr = %p\n", (void*)cpu0paddr);
 
 	/*
 	 * Now the kernel text segment is in its final location we can try to
@@ -1192,10 +1198,13 @@ pmap_bootstrap(u_long kernelstart, u_long kernelend)
 		cpus->ci_eintstack = NULL;
 		cpus->ci_spinup = main; /* Call main when we're running. */
 		cpus->ci_paddr = cpu0paddr;
+		printf("pmap: ci_paddr = %p\n", (void*)cpus->ci_paddr);
 		if (CPU_ISSUN4V) {
 			cpus->ci_mmfsa = cpu0paddr;
 			cpus->ci_tsb_desc = NULL;
 		}
+		cpus->ci_paddr += CPUINFO_VA - INTSTACK;
+		printf("pmap: ci_paddr AJUSTED = %p\n", (void*)cpus->ci_paddr);
 		cpus->ci_cpcb = (struct pcb *)u0va;
 		cpus->ci_idepth = -1;
 		memset(cpus->ci_intrpending, -1, sizeof(cpus->ci_intrpending));
